@@ -7,15 +7,16 @@ from azure.cli.core.commands import CliCommandType
 
 from ._format import (
     db_list_transform,
-    db_show_transform,
-    db_list_table_format,
-    db_show_table_format,
-    db_edition_list_table_format,
+    db_transform,
+    db_table_format,
+    db_edition_table_format,
     elastic_pool_list_transform,
-    elastic_pool_show_transform,
-    elastic_pool_list_table_format,
-    elastic_pool_show_table_format,
-    elastic_pool_edition_list_table_format,
+    elastic_pool_transform,
+    elastic_pool_table_format,
+    elastic_pool_edition_table_format,
+    firewall_rule_table_format,
+    server_table_format,
+    usage_table_format,
     LongRunningOperationResultTransform,
 )
 
@@ -32,6 +33,7 @@ from ._util import (
     get_sql_elastic_pools_operations,
     get_sql_elastic_pool_operations_operations,
     get_sql_encryption_protectors_operations,
+    get_sql_failover_groups_operations,
     get_sql_firewall_rules_operations,
     get_sql_managed_databases_operations,
     get_sql_managed_instances_operations,
@@ -66,9 +68,11 @@ def load_command_table(self, _):
                             subscription_usages_operations,
                             client_factory=get_sql_subscription_usages_operations) as g:
 
-        g.command('list-usages', 'list_by_location')
+        g.command('list-usages', 'list_by_location',
+                  table_transformer=usage_table_format)
 
-        g.command('show-usage', 'get')
+        g.command('show-usage', 'get',
+                  table_transformer=usage_table_format)
 
     ###############################################
     #                sql db                       #
@@ -82,7 +86,7 @@ def load_command_table(self, _):
         client_factory=get_sql_databases_operations)
 
     database_lro_transform = LongRunningOperationResultTransform(
-        self.cli_ctx, db_show_transform)
+        self.cli_ctx, db_transform)
 
     with self.command_group('sql db',
                             database_operations,
@@ -90,27 +94,33 @@ def load_command_table(self, _):
 
         g.custom_command('create', 'db_create',
                          supports_no_wait=True,
-                         transform=database_lro_transform)
+                         transform=database_lro_transform,
+                         table_transformer=db_table_format)
         g.custom_command('copy', 'db_copy',
                          supports_no_wait=True,
-                         transform=database_lro_transform)
+                         transform=database_lro_transform,
+                         table_transformer=db_table_format)
         g.custom_command('restore', 'db_restore',
-                         supports_no_wait=True, transform=database_lro_transform)
+                         supports_no_wait=True,
+                         transform=database_lro_transform,
+                         table_transformer=db_table_format)
         g.custom_command('rename', 'db_rename',
-                         transform=database_lro_transform)
-        g.command('show', 'get',
-                  transform=db_show_transform,
-                  table_transformer=db_show_table_format)
+                         transform=database_lro_transform,
+                         table_transformer=db_table_format)
+        g.show_command('show', 'get',
+                       transform=db_transform,
+                       table_transformer=db_table_format)
         g.custom_command('list', 'db_list',
                          transform=db_list_transform,
-                         table_transformer=db_list_table_format)
+                         table_transformer=db_table_format)
         g.command('delete', 'delete',
                   confirmation=True,
                   supports_no_wait=True)
         g.generic_update_command('update',
                                  custom_func_name='db_update',
                                  supports_no_wait=True,
-                                 transform=database_lro_transform)
+                                 transform=database_lro_transform,
+                                 table_transformer=db_table_format)
         g.custom_command('import', 'db_import')
         g.custom_command('export', 'db_export')
 
@@ -125,7 +135,7 @@ def load_command_table(self, _):
         g.custom_command(
             'list-editions',
             'db_list_capabilities',
-            table_transformer=db_edition_list_table_format)
+            table_transformer=db_edition_table_format)
 
     with self.command_group('sql db replica',
                             database_operations,
@@ -133,7 +143,8 @@ def load_command_table(self, _):
 
         g.custom_command('create', 'db_create_replica',
                          supports_no_wait=True,
-                         transform=database_lro_transform)
+                         transform=database_lro_transform,
+                         table_transformer=db_table_format)
 
     with self.command_group('sql dw',
                             database_operations,
@@ -142,12 +153,13 @@ def load_command_table(self, _):
         g.custom_command('create', 'dw_create',
                          supports_no_wait=True,
                          transform=database_lro_transform)
-        g.command('show', 'get',
-                  transform=db_show_transform)
+        g.show_command('show', 'get',
+                       transform=db_transform)
         g.custom_command('list', 'dw_list',
                          transform=db_list_transform)
         g.command('delete', 'delete',
-                  confirmation=True, supports_no_wait=True)
+                  confirmation=True,
+                  supports_no_wait=True)
         g.custom_command('pause', 'dw_pause')
         g.custom_command('resume', 'dw_resume')
         g.generic_update_command('update',
@@ -171,7 +183,7 @@ def load_command_table(self, _):
     with self.command_group('sql db tde', transparent_data_encryptions_operations) as g:
 
         g.command('set', 'create_or_update')
-        g.command('show', 'get')
+        g.show_command('show', 'get')
 
     transparent_data_encryption_activities_operations = CliCommandType(
         operations_tmpl='azure.mgmt.sql.operations.transparent_data_encryption_activities_operations#TransparentDataEncryptionActivitiesOperations.{}',
@@ -210,7 +222,7 @@ def load_command_table(self, _):
                             database_blob_auditing_policies_operations,
                             client_factory=get_sql_database_blob_auditing_policies_operations) as g:
 
-        g.command('show', 'get')
+        g.show_command('show', 'get')
         g.generic_update_command('update',
                                  custom_func_name='db_audit_policy_update')
 
@@ -222,7 +234,7 @@ def load_command_table(self, _):
                             database_threat_detection_policies_operations,
                             client_factory=get_sql_database_threat_detection_policies_operations) as g:
 
-        g.command('show', 'get')
+        g.show_command('show', 'get')
         g.generic_update_command('update',
                                  custom_func_name='db_threat_detection_policy_update')
 
@@ -243,7 +255,7 @@ def load_command_table(self, _):
         client_factory=get_sql_elastic_pools_operations)
 
     elastic_pool_lro_transform = LongRunningOperationResultTransform(
-        self.cli_ctx, elastic_pool_show_transform)
+        self.cli_ctx, elastic_pool_transform)
 
     with self.command_group('sql elastic-pool',
                             elastic_pools_ops,
@@ -251,32 +263,34 @@ def load_command_table(self, _):
 
         g.custom_command('create', 'elastic_pool_create',
                          supports_no_wait=True,
-                         transform=elastic_pool_lro_transform)
+                         transform=elastic_pool_lro_transform,
+                         table_transformer=elastic_pool_table_format)
         g.command('delete', 'delete',
                   supports_no_wait=True)
-        g.command('show', 'get',
-                  transform=elastic_pool_show_transform,
-                  table_transformer=elastic_pool_show_table_format)
+        g.show_command('show', 'get',
+                       transform=elastic_pool_transform,
+                       table_transformer=elastic_pool_table_format)
         g.command('list', 'list_by_server',
                   transform=elastic_pool_list_transform,
-                  table_transformer=elastic_pool_list_table_format)
+                  table_transformer=elastic_pool_table_format)
         g.generic_update_command('update',
                                  custom_func_name='elastic_pool_update',
                                  supports_no_wait=True,
-                                 transform=elastic_pool_lro_transform)
+                                 transform=elastic_pool_lro_transform,
+                                 table_transformer=elastic_pool_table_format)
 
     with self.command_group('sql elastic-pool', database_operations) as g:
 
         g.command('list-dbs', 'list_by_elastic_pool',
                   transform=db_list_transform,
-                  table_transformer=db_list_table_format)
+                  table_transformer=db_table_format)
 
     with self.command_group('sql elastic-pool',
                             capabilities_operations,
                             client_factory=get_sql_capabilities_operations) as g:
 
         g.custom_command('list-editions', 'elastic_pool_list_capabilities',
-                         table_transformer=elastic_pool_edition_list_table_format)
+                         table_transformer=elastic_pool_edition_table_format)
 
     elastic_pool_operations_operations = CliCommandType(
         operations_tmpl='azure.mgmt.sql.operations.elastic_pool_operations#ElasticPoolOperations.{}',
@@ -289,6 +303,21 @@ def load_command_table(self, _):
         g.command('list', 'list_by_elastic_pool')
         g.command('cancel', 'cancel')
 
+    ###############################################
+    #             sql failover-group              #
+    ###############################################
+
+    failover_groups_operations = CliCommandType(
+        operations_tmpl='azure.mgmt.sql.operations.failover_groups_operations#FailoverGroupsOperations.{}',
+        client_factory=get_sql_failover_groups_operations)
+    with self.command_group('sql failover-group', failover_groups_operations, client_factory=get_sql_failover_groups_operations) as g:
+        g.command('show', 'get')
+        g.command('list', 'list_by_server')
+        g.custom_command('create', 'failover_group_create')
+        g.generic_update_command('update', custom_func_name='failover_group_update')
+        g.command('delete', 'delete')
+        g.custom_command('set-primary', 'failover_group_failover')
+
     ###############################################
     #                sql server                   #
     ###############################################
@@ -301,11 +330,14 @@ def load_command_table(self, _):
                             servers_operations,
                             client_factory=get_sql_servers_operations) as g:
 
-        g.custom_command('create', 'server_create')
+        g.custom_command('create', 'server_create',
+                         table_transformer=server_table_format)
         g.command('delete', 'delete',
                   confirmation=True)
-        g.command('show', 'get')
-        g.custom_command('list', 'server_list')
+        g.show_command('show', 'get',
+                       table_transformer=server_table_format)
+        g.custom_command('list', 'server_list',
+                         table_transformer=server_table_format)
         g.generic_update_command('update',
                                  custom_func_name='server_update')
 
@@ -314,7 +346,6 @@ def load_command_table(self, _):
         client_factory=get_sql_server_usages_operations)
 
     with self.command_group('sql server', server_usages_operations) as g:
-
         g.command('list-usages', 'list_by_server')
 
     firewall_rules_operations = CliCommandType(
@@ -325,11 +356,15 @@ def load_command_table(self, _):
                             firewall_rules_operations,
                             client_factory=get_sql_firewall_rules_operations) as g:
 
-        g.command('create', 'create_or_update')
-        g.custom_command('update', 'firewall_rule_update')
+        g.command('create', 'create_or_update',
+                  table_transformer=firewall_rule_table_format)
+        g.custom_command('update', 'firewall_rule_update',
+                         table_transformer=firewall_rule_table_format)
         g.command('delete', 'delete')
-        g.command('show', 'get')
-        g.command('list', 'list_by_server')
+        g.show_command('show', 'get',
+                       table_transformer=firewall_rule_table_format)
+        g.command('list', 'list_by_server',
+                  table_transformer=firewall_rule_table_format)
 
     aadadmin_operations = CliCommandType(
         operations_tmpl='azure.mgmt.sql.operations.server_azure_ad_administrators_operations#ServerAzureADAdministratorsOperations.{}',
@@ -355,7 +390,7 @@ def load_command_table(self, _):
 
         g.custom_command('create', 'server_key_create')
         g.custom_command('delete', 'server_key_delete')
-        g.custom_command('show', 'server_key_get')
+        g.custom_show_command('show', 'server_key_get')
         g.command('list', 'list_by_server')
 
     encryption_protectors_operations = CliCommandType(
@@ -366,7 +401,7 @@ def load_command_table(self, _):
                             encryption_protectors_operations,
                             client_factory=get_sql_encryption_protectors_operations) as g:
 
-        g.command('show', 'get')
+        g.show_command('show', 'get')
         g.custom_command('set', 'encryption_protector_update')
 
     virtual_network_rules_operations = CliCommandType(
@@ -379,7 +414,7 @@ def load_command_table(self, _):
 
         g.command('create', 'create_or_update',
                   validator=validate_subnet)
-        g.command('show', 'get')
+        g.show_command('show', 'get')
         g.command('list', 'list_by_server')
         g.command('delete', 'delete')
         g.generic_update_command('update')
@@ -392,7 +427,7 @@ def load_command_table(self, _):
                             server_connection_policies_operations,
                             client_factory=get_sql_server_connection_policies_operations) as c:
 
-        c.command('show', 'get')
+        c.show_command('show', 'get')
         c.generic_update_command('update')
 
     server_dns_aliases_operations = CliCommandType(
@@ -403,7 +438,7 @@ def load_command_table(self, _):
                             server_dns_aliases_operations,
                             client_factory=get_sql_server_dns_aliases_operations) as c:
 
-        c.command('show', 'get')
+        c.show_command('show', 'get')
         c.command('list', 'list_by_server')
         c.command('create', 'create_or_update')
         c.command('delete', 'delete')
@@ -423,7 +458,7 @@ def load_command_table(self, _):
 
         g.custom_command('create', 'managed_instance_create', supports_no_wait=True)
         g.command('delete', 'delete', confirmation=True, supports_no_wait=True)
-        g.command('show', 'get')
+        g.show_command('show', 'get')
         g.custom_command('list', 'managed_instance_list')
         g.generic_update_command('update', custom_func_name='managed_instance_update', supports_no_wait=True)
 
@@ -441,6 +476,6 @@ def load_command_table(self, _):
 
         g.custom_command('create', 'managed_db_create', supports_no_wait=True)
         g.custom_command('restore', 'managed_db_restore', supports_no_wait=True)
-        g.command('show', 'get')
+        g.show_command('show', 'get')
         g.command('list', 'list_by_instance')
         g.command('delete', 'delete', confirmation=True, supports_no_wait=True)
